@@ -82,10 +82,10 @@ from cylc.flow.remote import (
     construct_cylc_server_ssh_cmd,
     construct_ssh_cmd,
 )
+from cylc.flow.rundb import CylcWorkflowDAO
 from cylc.flow.terminal import parse_dirty_json
 from cylc.flow.unicode_rules import WorkflowNameValidator
 from cylc.flow.util import cli_format
-from cylc.flow.workflow_db_mgr import WorkflowDatabaseManager
 
 if TYPE_CHECKING:
     from optparse import Values
@@ -203,6 +203,9 @@ class WorkflowFiles:
         VERSION = 'version'
         """Version control log dir"""
 
+        DB = 'db'
+        """The public database"""
+
     SHARE_DIR = 'share'
     """Workflow share directory."""
 
@@ -1200,7 +1203,7 @@ def get_workflow_title(reg):
     return title
 
 
-def get_platforms_from_db(run_dir):
+def get_platforms_from_db(run_dir: Path) -> Set[str]:
     """Return the set of names of platforms (that jobs ran on) from the DB.
 
     Warning:
@@ -1213,16 +1216,16 @@ def get_platforms_from_db(run_dir):
         compatibility. We can't apply upgraders which don't exist yet.
 
     Args:
-        run_dir (str): The workflow run directory.
+        run_dir: The workflow run directory.
 
     Raises:
         sqlite3.OperationalError: in the event the table/field required for
         cleaning is not present.
 
     """
-    workflow_db_mgr = WorkflowDatabaseManager(
-        os.path.join(run_dir, WorkflowFiles.Service.DIRNAME))
-    with workflow_db_mgr.get_pri_dao() as pri_dao:
+    with CylcWorkflowDAO(
+        run_dir / WorkflowFiles.Service.DIRNAME / WorkflowFiles.Service.DB
+    ) as pri_dao:
         platform_names = pri_dao.select_task_job_platforms()
 
     return platform_names
@@ -1633,7 +1636,7 @@ def install_workflow(
     run_name: Optional[str] = None,
     no_run_name: bool = False,
     cli_symlink_dirs: Optional[Dict[str, Dict[str, Any]]] = None
-) -> Tuple[Path, Path, str]:
+) -> Tuple[Path, Path, str, str]:
     """Install a workflow, or renew its installation.
 
     Install workflow into new run directory.
@@ -1655,6 +1658,7 @@ def install_workflow(
         rundir: absolute path to run directory, where the workflow has been
             installed into.
         workflow_name: installed workflow name (which may be computed here).
+        named_run: Name of the run.
 
     Raise:
         WorkflowFilesError:
@@ -1751,7 +1755,7 @@ def install_workflow(
     install_log.info(f'INSTALLED {named_run} from {source}')
     print(f'INSTALLED {named_run} from {source}')
     close_log(install_log)
-    return source, rundir, workflow_name
+    return source, rundir, workflow_name, named_run
 
 
 def get_run_dir_info(
